@@ -106,9 +106,9 @@ we still hold it as a string.
         
     def fromString(self,treeStr):
         '''Populate attributes by parsing the string treeStr (which has likely
-been read from a file). This method allows multifurcating trees. See
-fileStr method for description of format.
-
+been read from a file, and is our native file format for trees). This
+method allows multifurcating trees. See fileStr method for description
+of format.
         '''
         rootArbNode,nodeConnecStr,branchLenStr = treeStr.split("|")
 
@@ -508,9 +508,22 @@ preparation such as naming internal nodes).
         else:
             self.populateAttributes(nodeConnectD,bpTree.root.name)
 
+    def fromNewickFile(self,treeFN,includeBrLen=False):
+        '''Populate attributes based on newick file in treeFN. Assumes a rooted tree.
+        '''
+        bpTree = Phylo.read(treeFN, 'newick', rooted=True)
+        nodeConnectD,branchLenD = self.__bioPhyloToNodeConnectD__(bpTree)
+        if includeBrLen:
+            self.populateAttributes(nodeConnectD,bpTree.root.name,branchLenD)
+        else:
+            self.populateAttributes(nodeConnectD,bpTree.root.name)
+            
     def toNewickStr(self,includeBrLength=False,nodeLabelD=None):
         '''Output a newick string.'''
-        return self.__traverseForNewickStr__(self.rootNode,ROOT_PARENT_NAME,includeBrLength,nodeLabelD)
+        if self.nodeConnectD == None:
+            return "()"
+        else:
+            return self.__traverseForNewickStr__(self.rootNode,ROOT_PARENT_NAME,includeBrLength,nodeLabelD)
         
     def subtree(self,node):
         '''Return a new Rtree object with the subtree rooted at node.'''
@@ -605,6 +618,28 @@ preparation such as naming internal nodes).
             mrca = self.findMrcaPair(mrca,node)
         return mrca
 
+    def calcDistanceBetweenNodes(self, ancNode, descNode):
+        ''' Takes in two nodes in the tree and calculates the distance 
+        assuming that the first node is ancestral'''
+        if ancNode == descNode:
+            return 0
+        else:
+            currentNode = descNode
+            pairsLeadingToAnc = []
+            while currentNode != ancNode:
+                if currentNode == self.rootNode:
+                    return float('inf')
+                higherNode = self.getParent(currentNode)
+                pairsLeadingToAnc.append((currentNode,higherNode))
+                currentNode = higherNode
+            distSum = 0
+            for pair in pairsLeadingToAnc:
+                if pair in self.branchLenD:
+                    distSum += self.branchLenD[pair]
+                else:
+                    distSum += self.branchLenD[(pair[1],pair[0])]
+            return distSum
+
     def prune(self,leafL):
         '''Prune down to a tree with only the leaves in leafL, combining
 branches and removing nodes as needed. Returns a new Rtree.
@@ -849,7 +884,10 @@ named interal nodes (we will create those).
 
     def toNewickStr(self,includeBrLength=False,nodeLabelD=None):
         '''Output a newick string.'''
-        if self.nodeCount() == 1:
+        if self.nodeConnectD == None:
+            # empty
+            return "()"
+        elif self.nodeCount() == 1:
             return "("+self.leafNodeT[0]+")"
         elif self.nodeCount() == 2:
             # two tip tree is a special case            
@@ -1018,7 +1056,79 @@ node.'''
             aUtreeO = subUtree(self.nodeConnectD,self.branchPairT,self.branchLenD,branchPair[1],branchPair[0])
             bUtreeO = subUtree(self.nodeConnectD,self.branchPairT,self.branchLenD,branchPair[0],branchPair[1])
             return aUtreeO,bUtreeO
-        
+
+    def makeDistanceMatrix(self) -> dict:
+        """ makeDistanceMatrix:
+                Accepts a Utree object as input. Constructs and returns a matrix 
+                of the cophenetic distances for all of the pairwise combinations of
+                the tips. Returns the matrix as a dictionary keyed by tuples conta-
+                ining each pairwise combination.
+        """
+        # get all tips as a list
+        tipsL = list(self.leafNodeT)
+        # build a dictionary of all pairwise combinations
+        tableD = dict()
+        for tip1 in tipsL:
+            for tip2 in tipsL:
+                # self-vs-self comparisons are on the diagonal; they should be zero
+                if tip1 == tip2:
+                    distance = 0.0
+                # calculate the cophenetic distances for each comparison
+                else:
+                    # calculate the distance between the two tips
+                    path = self.__treePathfinder__(tip1, tip2)
+                    distance = self.__sumBranchLengths__(path)
+                # save the calculated distance in the tableD
+                tableD[(tip1, tip2)] = distance
+        return tableD
+    
+    def __treePathfinder__(self, origin:str, end:str, previous:str=None) -> list:
+        """ __treePathfinder__:
+                Accepts a Utree object, a string indicating the starting node/tip,
+                a string indicating the ending node/tip, and a string indicating 
+                the previous origin (used for recursion) as inputs. Recurses throu-
+                gh the the tree to find the branches (indicated by pairs of nodes)
+                that will connect the origin to the end within the tree. Returns a 
+                list of tuples where each tuple contains a pair of nodes.
+        """
+        # get all the nodes/tips that are connected to the origin
+        allConnxns = self.nodeConnectD[origin]
+         # base case: the end is connected to the origin
+        if end in allConnxns:
+            return [(origin, end)]
+        # iterate through the origin's connections
+        outL = list()
+        for connxn in allConnxns:
+            # only process connections that aren't leaves or the previous node
+            if not self.isLeaf(connxn) and connxn != previous:
+                # recurse on connection towards the specified 'end'
+                path = self.__treePathfinder__(connxn, end, previous=origin)
+                # if a path was found ...
+                if path != []:
+                    # ... then append the origin and its connection to the list ...
+                    outL.append((origin, connxn))
+                    # ... and add the path that was found to the list
+                    outL += path
+        # returns an empty list if nothing was found or if nothing to do
+        return outL
+
+    def __sumBranchLengths__(self, pathL:list) -> float:
+        """ __sumBranchLengths__:
+                Accepts a Utree object and a list of node-pairs (tuples) as inputs.
+                Calculates and returns the total distance represented by the list
+                of node-pairs.
+        """
+        # initialize the sum
+        totalLen = 0.0
+        # for each node-pair in the list
+        for nodePair in pathL:
+            # flip the order if the pair is not a key in branchLenD
+            if nodePair not in self.branchLenD.keys():
+                nodePair = (nodePair[1], nodePair[0])
+            # add the length to the running total
+            totalLen += self.branchLenD[nodePair]
+        return totalLen
+
     def __bioPhyloToNodeConnectD__(self,bpClade,parentNodeStr,nodeConnectD,iNodeNum):
         '''Convert a biopython clade object to a node connection dict with
 keys that are nodes, and values that are the nodes connected to. For
@@ -1037,16 +1147,18 @@ named internal nodes, and we name them here: g0, g1 etc.
             # operate over this branch, joining the two nodes with no
             # node in between.
 
+            # get branch length for new branch with root removed
+            if bpClade[0].branch_length == None or bpClade[1].branch_length == None:
+                brLen = None
+            else:
+                brLen = bpClade[0].branch_length + bpClade[1].branch_length
+            
+            # handle cases where one or two are terminal
             if bpClade[0].is_terminal() and bpClade[1].is_terminal():
                 # both terminal
                 iNodeNum,nodeConnectD,_ = self.__bioPhyloToNodeConnectD__(bpClade[0],bpClade[1].name,nodeConnectD,iNodeNum)
                 iNodeNum,nodeConnectD,_ = self.__bioPhyloToNodeConnectD__(bpClade[1],bpClade[0].name,nodeConnectD,iNodeNum)
                 
-                # get branches
-                if bpClade[0].branch_length == None or bpClade[1].branch_length == None:
-                    brLen = None
-                else:
-                    brLen = bpClade[0].branch_length + bpClade[1].branch_length
                 branchLenL = [(bpClade[0].name,bpClade[1].name,brLen)]
                 
             elif bpClade[0].is_terminal():
@@ -1055,11 +1167,22 @@ named internal nodes, and we name them here: g0, g1 etc.
                 iNodeNum,nodeConnectD,_ = self.__bioPhyloToNodeConnectD__(bpClade[0],"g"+str(iNodeNum),nodeConnectD,iNodeNum)
                 iNodeNum,nodeConnectD,branchLenL = self.__bioPhyloToNodeConnectD__(bpClade[1],bpClade[0].name,nodeConnectD,iNodeNum)
 
+                # fix len of br made when root removed
+                for i in range(len(branchLenL)):
+                    if bpClade[0].name in branchLenL[i]:
+                        # only be 1 branch w/this in it. Update the
+                        # len to proper one calculated above
+                        branchLenL[i] = (branchLenL[i][0],branchLenL[i][1],brLen)
+                
             elif bpClade[1].is_terminal():
                 # 1 is terminal, 0 not
                 # parent node for 1 will be based on current iNodeNum
                 iNodeNum,nodeConnectD,_ = self.__bioPhyloToNodeConnectD__(bpClade[1],"g"+str(iNodeNum),nodeConnectD,iNodeNum)
                 iNodeNum,nodeConnectD,branchLenL = self.__bioPhyloToNodeConnectD__(bpClade[0],bpClade[1].name,nodeConnectD,iNodeNum)
+                # fix len of br made when root removed
+                for i in range(len(branchLenL)):
+                    if bpClade[1].name in branchLenL[i]:
+                        branchLenL[i] = (branchLenL[i][0],branchLenL[i][1],brLen)
 
             else:
                 # neither is terminal
@@ -1083,7 +1206,13 @@ named internal nodes, and we name them here: g0, g1 etc.
                 # include in branchLenL. The entry that has
                 # ("g"+str(iNodeNumAtBaseOf0),"g"+str(iNodeNumAtBaseOf1),brLen)
                 for brLenT in tempBranchLenL:
-                    if brLenT[0] != "g"+str(iNodeNumAtBaseOf0) or brLenT[1] != "g"+str(iNodeNumAtBaseOf1):
+                    if brLenT[0] == "g"+str(iNodeNumAtBaseOf0) and brLenT[1] == "g"+str(iNodeNumAtBaseOf1):
+                        pass
+                    elif brLenT[1] == "g"+str(iNodeNumAtBaseOf0) and brLenT[0] == "g"+str(iNodeNumAtBaseOf1):
+                        # keep this, updating len of br
+                        newBrLenT = (brLenT[1],brLenT[0],brLen) # brLen is correct len of br where root removed
+                        branchLenL.append(newBrLenT)
+                    else:
                         branchLenL.append(brLenT)
         else:
             # general case
