@@ -56,7 +56,7 @@ originFamilies object.
     writeFamilies(initialFamiliesO,initFamilyFN,genesO,strainNamesT,paramD)
     print("Initial families:",file=outputSummaryF)
     writeFamilyFormationSummary(initialFamiliesO,outputSummaryF)
-    
+
     # reconcile
     initialFamiliesO = reconcileGeneTrees(initialFamiliesO.iterFamilies(),speciesRtreeO,initialFamiliesO,locusMapD,genesO,numProcesses,D,T,L,O,R)
 
@@ -73,6 +73,10 @@ originFamilies object.
     writeFamilies(initialFamiliesO,initFamilyFN,genesO,strainNamesT,paramD) # update with recons
     writeFamilies(originFamiliesO,originFamilyFN,genesO,strainNamesT,paramD)
 
+    # optionally delete gene trees work dir
+    if paramD['deleteGeneFamilyTreesDir']:
+        shutil.rmtree(os.path.join(paramD['geneFamilyTreesDir']))
+    
     return initialFamiliesO,originFamiliesO
 
 ## Support functions
@@ -103,6 +107,10 @@ def homologyCheck(genesO,aabrhHardCoreL,scoresO,outputSummaryF,paramD):
     homologousPeakMissingL = []
     for strainPair in scoresO.getStrainPairs():
 
+        if strainPair[0] == strainPair[1]:
+            # strain vs self, skip
+            continue
+        
         scoreIterator = scoresO.iterateScoreByStrainPair(strainPair,'rawSc')
         binHeightL,indexToBinCenterL = scoreHist(scoreIterator,scoreHistNumBins)
         homologPeakLeftExtremePos=homologPeakChecker(binHeightL,indexToBinCenterL,binWidth,paramD)
@@ -212,6 +220,7 @@ def createBlastFamilies(paramD,speciesRtreeO,strainNamesT,scoresO,genesO,outputS
     blastFamGeneTreeFileStem = paramD['blastFamGeneTreeFileStem']
     blastFamilyFN = paramD['blastFamilyFN']
     maxBlastFamSize = int(paramD['maxBlastFamSizeMultiplier'] * speciesRtreeO.leafCount())
+    GeneRaxOutputDirN = paramD['GeneRaxOutputDirN']+"-"+blastFamGeneTreeFileStem
     
     ## get blast families as list of sets
     print("Blast families:",file=outputSummaryF)
@@ -235,15 +244,35 @@ def createBlastFamilies(paramD,speciesRtreeO,strainNamesT,scoresO,genesO,outputS
         
     # delete any pre-existing blast family trees
     blastFamGeneTreeFilePath = os.path.join(geneFamilyTreesDir,blastFamGeneTreeFileStem+'*.tre')
+
     for fn in glob.glob(blastFamGeneTreeFilePath):
         os.remove(fn)
 
-    trees.makeGeneTrees(paramD,False,genesO,geneFamilyTreesDir,blastFamGeneTreeFileStem,blastFamilySetL)
+    # make trees
+    if paramD['useGeneRaxToMakeSpeciesTrees']:
+        # use GeneRax
+        # run GeneRax on families of 3 genes or more. Run FastTree on rest
+        # (will put branch lengths on 2 species trees, which we want
+        geneRaxBlastFamilySetL = []
+        fastTreeBlastFamilySetL = []
+        for i,bSet in blastFamilySetL:
+            if len(bSet) > 2:
+                geneRaxBlastFamilySetL.append((i,bSet))
+            else:
+                fastTreeBlastFamilySetL.append((i,bSet))
+
+        failedGeneRaxOrthoSetL = trees.makeGeneTreesGeneRax(paramD,False,genesO,geneFamilyTreesDir,blastFamGeneTreeFileStem,geneRaxBlastFamilySetL)
+
+        fastTreeBlastFamilySetL.extend(failedGeneRaxOrthoSetL)
+    else:
+        # use FastTree for all
+        fastTreeBlastFamilySetL = blastFamilySetL
+        
+    trees.makeGeneTreesFastTree(paramD,False,genesO,geneFamilyTreesDir,blastFamGeneTreeFileStem,fastTreeBlastFamilySetL)
+
+    # collect and save blast family trees into a single file at the top level
+    collectBlastFamilyTreesToSingleFile(paramD)
 
-    # remove alignments
-    for fn in glob.glob(os.path.join(geneFamilyTreesDir,"align*.afa")):
-        os.remove(fn)
-    
 def createBlastFamilySetL(scoresO,genesO,strainNamesT,outputSummaryF,maxBlastFamSize):
     '''
     Input
@@ -487,7 +516,19 @@ set in connecL if none of the existing sets have either gene.
     newClusterS = set([gn1,gn2])
     splitClusterL.append(newClusterS)
     return splitClusterL
-            
+
+def collectBlastFamilyTreesToSingleFile(paramD):
+    '''Collect and save blast family trees into a single file at the top
+level.
+    '''
+    blastFamGeneTreeL = loadGeneTreesFromDir(paramD,paramD['blastFamGeneTreeFileStem'])
+    
+    with open(paramD['blastFamilyTreeFN'],'w') as f:
+        for famNum,geneUtreeO in blastFamGeneTreeL:
+            treeStr = geneUtreeO.toNewickStr(includeBrLength=True)
+            f.write(str(famNum)+'\t'+treeStr+'\n')
+    return
+
 ## Initial family formation    
 def createInitialFamiliesO(paramD,genesO,aabrhHardCoreL,scoresO,speciesRtreeO,outputSummaryF):
     ''''''
@@ -498,7 +539,7 @@ def createInitialFamiliesO(paramD,genesO,aabrhHardCoreL,scoresO,speciesRtreeO,ou
     forceSplitUtreeBalanceMultiplier = paramD['forceSplitUtreeBalanceMultiplier']
 
     ## load gene trees
-    blastFamGeneTreeL = loadGeneTrees(paramD,blastFamGeneTreeFileStem)
+    blastFamGeneTreeL = loadGeneTreesFromDir(paramD,blastFamGeneTreeFileStem)
 
     # delete any aabrh gene trees
     # aside: for aabrh trees made for species tree calculation, the
@@ -515,13 +556,10 @@ def createInitialFamiliesO(paramD,genesO,aabrhHardCoreL,scoresO,speciesRtreeO,ou
     for orthoT in aabrhHardCoreL:
         newAabrhHardCoreL.append((orthoNum,orthoT))
         orthoNum += 1
-    trees.makeGeneTrees(paramD,False,genesO,geneFamilyTreesDir,aabrhHardCoreGeneTreeFileStem,newAabrhHardCoreL)
-    
-    aabrhHardCoreGeneTreeL = loadGeneTrees(paramD,aabrhHardCoreGeneTreeFileStem)
 
-    # remove alignments
-    for fn in glob.glob(os.path.join(geneFamilyTreesDir,"align*.afa")):
-        os.remove(fn)
+    trees.makeGeneTreesFastTree(paramD,False,genesO,geneFamilyTreesDir,aabrhHardCoreGeneTreeFileStem,newAabrhHardCoreL)
+    
+    aabrhHardCoreGeneTreeL = loadGeneTreesFromDir(paramD,aabrhHardCoreGeneTreeFileStem)
 
     ## split blast family trees
 
@@ -581,7 +619,7 @@ def createInitialFamiliesO(paramD,genesO,aabrhHardCoreL,scoresO,speciesRtreeO,ou
         
     return initialFamiliesO, locusMapD
 
-def loadGeneTrees(paramD,geneTreeFileStem):
+def loadGeneTreesFromDir(paramD,geneTreeFileStem):
     '''Load gene trees from the geneFamilyTreesDir that begin with
     geneTreeFileStem.
     '''
@@ -874,7 +912,8 @@ in that family, add the family to the object.
             locusMapD[gene]=locusFamNumCounter
             speciesL.append(genesO.numToStrainName(gene))
         lfMrca=speciesRtreeO.findMrca(speciesL)
-        lf=LocusFamily(famNumCounter,locusFamNumCounter,lfMrca)
+        # create a lf. for initial families, locusNum is the same as locusFamNum.
+        lf=LocusFamily(famNumCounter,locusFamNumCounter,lfMrca,locusNum=locusFamNumCounter)
         lf.addGenes(locusFamilyL, genesO)
         totalAddedTolocusFamilies+=len(locusFamilyL)
         initialFamiliesO.addLocusFamily(lf)
@@ -1464,7 +1503,7 @@ originFamiliesO.
 
     # run on multiple processors
     with Pool(processes=paramD['numProcesses']) as p:
-        for ifamNum,bestMprOrigFormatD in p.imap_unordered(getBestMprOrigForamatD, argumentL):
+        for ifamNum,bestMprOrigFormatD in p.imap_unordered(getBestMprOrigFormatD, argumentL):
             # put output bestMprs back in ifam objects
             ifam = initialFamiliesO.getFamily(ifamNum)
             ifam.addMprD(bestMprOrigFormatD)
@@ -1491,7 +1530,7 @@ originFamiliesO.
 
     return initialFamiliesO,originFamiliesO
 
-def getBestMprOrigForamatD(argumentL):
+def getBestMprOrigFormatD(argumentL):
     '''Given a candidate ifam, find the best MPR given nearbyOfamL.'''
 
     # Note: passing the largish objects in argumentL could become a problem with large datasets
