@@ -1,5 +1,5 @@
 """Provides the entry point to xenoGI's functionality."""
-__version__ = "3.0.0"
+__version__ = "3.1.0"
 import sys, glob, os, readline, rlcompleter
 from . import parameters,genbank,blast,trees,genomes,Score,scores,families,islands,analysis,islandBed
 from .Tree import *
@@ -12,14 +12,14 @@ def main():
         assert(len(sys.argv) == 3)
         paramFN=sys.argv[1]
         task = sys.argv[2]
-        assert(task in ['parseGenbank', 'runBlast', 'calcScores','makeSpeciesTree', 'makeFamilies', 'makeIslands','refine', 'printAnalysis', 'createIslandBed', 'plotScoreHists', 'interactiveAnalysis', 'runAll', 'version', 'debug'])
+        assert(task in ['parseGenbank', 'runBlast', 'calcScores','makeSpeciesTree', 'makeFamilies', 'makeIslands','refine', 'printAnalysis', 'interactiveAnalysis', 'createIslandBed', 'plotScoreHists', 'aminoAcidIdentity', 'runAll', 'version', 'debug'])
     
     except:
         print(
             """
    Exactly two arguments required.
       1. A path to a parameter file.
-      2. The task to be run which must be one of: parseGenbank, runBlast, calcScores, makeSpeciesTree, makeFamilies, makeIslands, refine, printAnalysis, createIslandBed, plotScoreHists, interactiveAnalysis, runAll or version.
+      2. The task to be run which must be one of: parseGenbank, runBlast, calcScores, makeSpeciesTree, makeFamilies, makeIslands, refine, printAnalysis, interactiveAnalysis, createIslandBed, plotScoreHists, aminoAcidIdentity, runAll or version.
 
    For example: 
       xenoGI params.py parseGenbank
@@ -64,17 +64,24 @@ def main():
     elif task == 'printAnalysis':
         printAnalysisWrapper(paramD,paramD['speciesTreeFN'],paramD['rootFocalClade'])
 
-    #### createIslandBed
-    elif task == 'createIslandBed':
-        createIslandBedWrapper(paramD)
-
     #### interactiveAnalysis
     elif task == 'interactiveAnalysis':
         interactiveAnalysisWrapper(paramD)
         
+    #### createIslandBed
+    elif task == 'createIslandBed':
+        createIslandBedWrapper(paramD)
+
     #### plotScoreHists
     elif task == 'plotScoreHists':
         scores.plotScoreHists(paramD)
+
+    #### aminoAcidIdentity
+    elif task == 'aminoAcidIdentity':
+        '''Assumes runBlast has already been run.'''
+        strainNamesT = readStrainInfoFN(paramD['strainInfoFN'])
+        aaiD=analysis.calculateAAI(paramD,strainNamesT)
+        analysis.printAminoAcidIdentity(aaiD,strainNamesT)
         
     #### runAll
     elif task == 'runAll':
@@ -96,10 +103,6 @@ def main():
     elif task == 'debug':
         debugWrapper(paramD)
 
-    #### validation via simulation
-    elif task == 'simValidation':
-        simValidationWrapper(paramD)
-        
 ######## Task related functions
 
 def parseGenbankWrapper(paramD):
@@ -156,7 +159,7 @@ def calcScoresWrapper(paramD,blastFnL):
     
     # object for storing scores
     scoresO=Score.Score()
-    scoresO.initializeDataAttributes(blastFnL,paramD,strainNamesT)
+    scoresO.initializeDataAttributes(blastFnL,paramD,strainNamesT,genesO)
 
     ## similarity scores
     scoresO = scores.calcRawScores(paramD,scoresO)
@@ -291,13 +294,13 @@ def interactiveAnalysisWrapper(paramD):
 
     ## Set up the modules a bit differently for interactive mode
     import code,sys
-    from xenoGI.analysis import createGene2FamIslandD,printScoreMatrix,matchFamilyIsland,printLocusIslandNeighb,vPrintLocusIslandsAtNode,printOutsideFamilyScores
+    from xenoGI.analysis import createGene2FamIslandD,printScoreMatrix,matchFamilyIsland,printLocusIslandNeighb,vPrintLocusIslandsAtNode,printOutsideFamilyScores,getOFamsWithEvent,countAllEvents
     from .xenoGI import families
     
     ## Wrapper analysis functions. For convenience these assume a
     ## bunch of global variables.
     
-    def printFam(familyNum,familiesO,fileF=sys.stdout):
+    def printFam(familiesO,familyNum,fileF=sys.stdout):
         '''This is a wrapper to provide an easy way to print relevant info on
     a family. For ease of use, we take only two arguments, assuming
     all the other required stuff is available at the top
@@ -309,8 +312,11 @@ def interactiveAnalysisWrapper(paramD):
         fam = familiesO.getFamily(familyNum)
         # print out the locus families
         for lfO in fam.getLocusFamilies():
-            print("    LocusFamily",lfO.getStr(genesO," "),file=fileF)
-
+            print("    LocusFamily",lfO.locusFamNum,file=fileF)
+            print("      lfMrca "+lfO.lfMrca,file=fileF)
+            for geneNum in lfO.iterGenes():
+                print("      "+genesO.numToName(geneNum),file=fileF)
+            
         if hasattr(fam,"sourceFam"):
             print(file=fileF)
             print("    Source family",fam.sourceFam,file=fileF)
@@ -339,14 +345,24 @@ def interactiveAnalysisWrapper(paramD):
         print(file=fileF)
 
         if hasattr(fam,"printReconByGeneTree"):
-            print("Gene tree",file=fileF)
-            print(fam.geneTreeO.toNewickStr(),file=fileF)
-            print()
-            print("Gene tree annotated with reconciliation [branch events | node events]",file=fileF)
-            print(fam.getNewickGeneTreeWithReconLabels(genesO),file=fileF)
-            print(file=fileF)
-            print("Reconciliation of gene tree onto species tree",file=fileF)       
-            fam.printReconByGeneTree(genesO,fileF)
+            # is ofam
+            ofam = fam
+        else:
+            # is ifam, we must convert dtlorMprD to node format before printing
+            speciesPreOrderT = speciesRtreeO.preorder()
+            dtlorMprD = fam.getMprReconDFromMpr(speciesPreOrderT,paramD)
+            ofam = originFamily(None,None,geneTreeO=fam.geneTreeO,dtlorMprD=dtlorMprD)
+            ofam.printReconByGeneTree(genesO)
+
+        print("Gene tree",file=fileF)
+        print(ofam.geneTreeO.toNewickStr(),file=fileF)
+        print()
+        print("Gene tree annotated with reconciliation [branch events | node events]",file=fileF)
+        print(ofam.getNewickGeneTreeWithReconLabels(genesO,includeBrLength=True),file=fileF)
+        print(file=fileF)
+        print("Reconciliation of gene tree onto species tree",file=fileF)       
+        ofam.printReconByGeneTree(genesO,fileF)
+            
         
     def findGene(searchStr,fileF=sys.stdout):
         '''Find information about a gene. Searches all the fields present in
@@ -376,6 +392,8 @@ def interactiveAnalysisWrapper(paramD):
         vPrintLocusIslandsAtNode(islandByNodeD[node],paramD['rootFocalClade'],subtreeD,originFamiliesO,genesO,fileF)
 
     ## Load data
+
+    print("Loading data, this may take some time...")
     
     strainNamesT,genesO,geneOrderD = loadGenomeRelatedData(paramD)
     speciesRtreeO,subtreeD = loadTreeRelatedData(paramD['speciesTreeFN'])
@@ -389,11 +407,12 @@ def interactiveAnalysisWrapper(paramD):
     scoresO.createNodeConnectD() # make nodeConnectD attribute
     
     # set up interactive console
-    vars = globals()
-    vars.update(locals())
-    readline.set_completer(rlcompleter.Completer(vars).complete)
+    variables = globals()
+    variables.update(locals())
+    readline.set_completer(rlcompleter.Completer(variables).complete)
     readline.parse_and_bind("tab: complete")
-    code.InteractiveConsole(vars).interact()
+    print("Ready for interactive analysis.")
+    code.InteractiveConsole(variables).interact()
 
 def debugWrapper(paramD):
     '''Take us into interactive mode for debugging.'''
@@ -402,15 +421,12 @@ def debugWrapper(paramD):
     import code,sys,numpy
     from .xenoGI import analysis,parameters,trees,genomes,families,islands,analysis,Score,scores
 
-    sys.path.insert(0,os.path.join(sys.path[0],'/biology/bush/htrans/xenoGI.work/misc/'))
-    import xlMode
-    
     strainNamesT,genesO,geneOrderD = loadGenomeRelatedData(paramD)
     speciesRtreeO,subtreeD = loadTreeRelatedData(paramD['speciesTreeFN'])
-   
+
     # set up interactive console
-    vars = globals()
-    vars.update(locals())
-    readline.set_completer(rlcompleter.Completer(vars).complete)
+    variables = globals()
+    variables.update(locals())
+    readline.set_completer(rlcompleter.Completer(variables).complete)
     readline.parse_and_bind("tab: complete")
-    code.InteractiveConsole(vars).interact()
+    code.InteractiveConsole(variables).interact()
